@@ -39,7 +39,7 @@ do_message(char *msg)
 	};
 	
 	about = gnome_about_new ( _("GULP"), VERSION,
-				  /* copyrigth notice */
+				  /* copyright notice */
 				  "(C) 1998 the Free Software Foundation",
 				  authors,
 			 /* another comments */
@@ -317,7 +317,7 @@ do_message(char *msg)
       printer_list = [[[[Gtk_CList alloc] initWithCListInfoTitles:4 Titles:columns] show] freeze];
       [printer_list set_policy:GTK_POLICY_ALWAYS :GTK_POLICY_AUTOMATIC];
       [printer_list set_selection_mode:GTK_SELECTION_BROWSE];
-      [printer_list connectObj:_("select_row") :self];
+      [printer_list connectObj:"select_row" :self];
       [printer_list set_column_width:0 :60];
       [printer_list set_column_width:1 :30];
       [printer_list set_column_width:2 :60];
