@@ -1,5 +1,5 @@
 /*
- * $Id: about.c,v 1.6 1998/07/04 15:58:36 gregm Exp $
+ * $Id: about.c,v 1.7 1998/08/10 23:09:40 gregm Exp $
  * GXSNMP -- An snmp mangament application
  * Copyright (C) 1998 Gregory McLean
  *
@@ -23,8 +23,6 @@
 static char const copyright[] =
 "@(#) Copyright (c) 1998 Gregory McLean";
 #endif
-static char const rcsid[] = 
-"$Id: about.c,v 1.6 1998/07/04 15:58:36 gregm Exp $";
 
 #include "main.h"
 
@@ -45,11 +43,13 @@ static void
 create_panel ()
 {
   GtkWidget   *about;
+  gchar       *about_pixmap_filename;
   gchar       *authors[] = {
     "Gregory McLean",
     "Jochen Friedrich",
     NULL
   };
+  about_pixmap_filename = gnome_unconditional_pixmap_file ("logo_h.xpm");
   about = gnome_about_new ( "GXSNMP", VERSION,
 			      "(C) 1998 Gregory McLean & Jochen Friedrich",
                               authors,
@@ -70,4 +70,3 @@ void open_about_panel ()
 }
 
 /* EOF */
-
