@@ -1,5 +1,5 @@
 /*
- * $Id: date.c,v 1.4 1998/04/16 14:49:14 jochen Exp $
+ * $Id: date.c,v 1.5 1998/06/14 21:18:32 gregm Exp $
  * GXSNMP - An snmp managment application
  * Copyright (C) 1998 Gregory McLean
  *
@@ -25,12 +25,12 @@ static char const copyright[] =
 "@(#) Copyright (C) 1998 Gregory McLean";
 #endif
 static char const rcsid[] =
-"$Id: date.c,v 1.4 1998/04/16 14:49:14 jochen Exp $";
+"$Id: date.c,v 1.5 1998/06/14 21:18:32 gregm Exp $";
 
 #include "main.h"
 
 char *
-sql_time_str (char *buf)
+db_time_str (char *buf)
 {
   struct tm     *tim;
   time_t        gm_time;
