@@ -1,5 +1,5 @@
 /*
- * $Id: db_interface.c,v 1.1 1998/05/25 18:31:53 gregm Exp $
+ * $Id: db_interface.c,v 1.3 1998/07/04 15:58:40 gregm Exp $
  * GXSNMP -- An snmp mangament application
  * Copyright (C) 1998 Gregory McLean
  *
@@ -25,6 +25,57 @@ static char const copyright[] =
 #endif
 
 #include "main.h"
+#include "db_interface.h"
+#ifdef HAVE_SQL
+#include <mysql/mysql.h>
+#endif
+
+/*
+ * Local functions
+ */
+
+/*
+ * Global Functions
+ */
+
+/*
+ * Call this before any db actions.
+ * This should take care of any db specific initilization.
+ */
+gboolean
+db_init ()
+{
+}
+
+gboolean
+db_connect (db_conn *connection)
+{
+#ifdef HAVE_SQL
+
+  connection->db_handle = (MYSQL *)g_malloc (sizeof(MYSQL));
+
+  if (mysql_connect (connection->db_handle, 
+		     connection->db_host,
+		     connection->db_user,
+		     connection->db_password) )
+    {
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
+    }
+#endif
+
+}
 
 /* EOF */
 
+
+
+
+
+
+
+
+
