@@ -1,5 +1,5 @@
 /*
- * $Id: host.h,v 1.2 1998/03/12 21:17:51 gregm Exp $
+ * $Id: host.h,v 1.3 1998/08/10 23:09:45 gregm Exp $
  * GXSNMP -- An snmp management application
  * Copyright (C) 1998 Gregory McLean
  *
@@ -24,7 +24,7 @@
 #include "snmp_host.h"
 
 struct hostent *lookuphost 	   (char *hostname);
-hosts          *find_host          (char *hostname);
+hosts          *find_host          (const char *hostname);
 hosts          *hl_add_host        (hosts *entry);
 void           hl_del_host         (hosts *entry);
 hosts          *get_current_host   (void);
