@@ -539,6 +539,7 @@ open_map_panel ()
       reset_map_panel ();
       test = (net_entry *) g_malloc (sizeof (net_entry));
       test->nl_name = g_strdup ("Test Network");
+      test->nl_hosts = NULL;
       map_add_network (test, 100, 50);
     }
 }
@@ -783,12 +784,13 @@ map_connect_host_net (hosts *host, net_entry *network)
   points->coords[2] = host_group->xpos;
   points->coords[3] = host_group->ypos;
 
-  host->hl_mconnection = gnome_canvas_item_new (GNOME_CANVAS_GROUP (GNOME_CANVAS_ITEM (host_group)->canvas->root),
-			 gnome_canvas_line_get_type (),
-			 "points", points,
-			 "fill_color", "black",
-			 "width_units", 0.0,
-			 NULL);
+  host->hl_mconnection = gnome_canvas_item_new (GNOME_CANVAS_GROUP 
+			 (GNOME_CANVAS_ITEM (host_group)->canvas->root),
+						gnome_canvas_line_get_type (),
+						"points", points,
+						"fill_color", "black",
+						"width_units", 0.0,
+						NULL);
   host->hl_ngroup = network->nl_mgroup;
   host->hl_network = network;
   add_host_to_network (network, host);
@@ -817,3 +819,4 @@ setup_host_net_connection (hosts *host, net_entry *network)
 }
 
 /* EOF */
+
