@@ -1,5 +1,5 @@
  /*
- * $Id: ping.c,v 1.3 1998/07/09 22:36:30 martin Exp $
+ * $Id: ping.c,v 1.4 1998/08/05 15:23:37 wilhelmi Exp $
  *                      P I N G . C
  *
  * Using the InterNet Control Message Protocol (ICMP) "ECHO" facility,
@@ -33,7 +33,6 @@
 #include <errno.h>
 #include <sys/param.h>
 #include <sys/file.h>
- /* #include <sys/filio.h> */
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
@@ -43,6 +42,9 @@
 /* #include <netinet/ip_var.h> */
 #include <gtk/gtk.h>
 #include "main.h"
+#ifdef HAVE_SYS_FILIO_H  /* needed for solaris */
+#  include <sys/filio.h>
+#endif
 
 #ifndef HAVE_STRUCT_ICMP
 /* This should only be the case on Linux Libc 5. */
