@@ -1,5 +1,5 @@
 /*
- * $Id: snmp_lib.c,v 1.13 1998/07/27 18:41:28 jochen Exp $
+ * $Id: snmp_lib.c,v 1.14 1998/08/06 19:14:52 jochen Exp $
  * GXSNMP -- An snmp management application
  * Copyright (C) 1998 Gregory McLean
  *
@@ -24,7 +24,7 @@ static char const copyright[] =
 "@(#) Copyright (c) 1998 Gregory McLean";
 #endif
 static char const rcsid[] =
-"$Id: snmp_lib.c,v 1.13 1998/07/27 18:41:28 jochen Exp $";
+"$Id: snmp_lib.c,v 1.14 1998/08/06 19:14:52 jochen Exp $";
 
 #include <config.h>               /* pull in the autoconf genereated stuff */
 #include <sys/types.h>
@@ -93,7 +93,8 @@ cb_register(guint socket, void (*receiveMessage)())
 void
 snmpinit() 
 {
-  g_snmp_init(cb_register);
+  if (!g_snmp_init(FALSE, cb_register))
+    g_error("Initialisation of SNMP library failed.");
   snmp_time = gtk_timeout_add (1000, g_snmp_timeout_cb, NULL);
 } 
 /* EOF */
