@@ -1,7 +1,7 @@
 /*
  * GXSNMP -- An snmp mangament application
  * Copyright (C) 1998 Gregory McLean & Jochen Friedrich
- * Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group
+ * Beholder RMON ethernet network monitor, Copyright (C) 1993 DNPAP group
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@
  *              g_asn1_open (asn1, buf_start, buf_len, ASN1_DEC);
  *              g_asn1_header_decode (asn1, &end_of_seq, cls, con, tag);
  *              g_asn1_header_decode (asn1, &end_of_octs, cls, con, tag);
- *              g_asn1_octets_decode (asn1, end_of_octs, str, sizeof(str), len);
+ *              g_asn1_octets_decode (asn1, end_of_octs, str, len);
  *              g_asn1_header_decode (asn1, &end_of_int, cls, con, tag);
  *              g_asn1_int_decode (asn1, end_of_int, &integer);
  *              g_asn1_eoc_decode (asn1, end_of_seq);
@@ -64,8 +64,6 @@
 
 #include "g_snmp.h"
 
-int g_asn1ErrStatus = ASN1_ERR_NOERROR;
-
 /*
  * NAME:        g_asn1_open                                   [API]
  * SYNOPSIS:    void g_asn1_open
@@ -92,6 +90,7 @@ g_asn1_open(ASN1_SCK *asn1, guchar *buf, guint len, guint mode)
     asn1->begin = buf;
     asn1->end = buf + len;
     asn1->pointer = (mode == ASN1_ENC) ? buf + len : buf;
+    asn1->error = ASN1_ERR_NOERROR;
 }
 
 /*
@@ -100,7 +99,7 @@ g_asn1_open(ASN1_SCK *asn1, guchar *buf, guint len, guint mode)
  *                  (
  *                      ASN1_SCK   *asn1,
  *                      guchar    **buf,
- *                      guint   *len
+ *                      guint      *len
  *                  )
  * DESCRIPTION: Closes an ASN1 socket.
  *              Parameters:
@@ -113,8 +112,8 @@ g_asn1_open(ASN1_SCK *asn1, guchar *buf, guint len, guint mode)
 void 
 g_asn1_close(ASN1_SCK *asn1, guchar **buf, guint *len)
 {
-    *buf = asn1->pointer;
-    *len = asn1->end - asn1->pointer;
+    *buf   = asn1->pointer;
+    *len   = asn1->end - asn1->pointer;
 }
 
 
@@ -135,7 +134,7 @@ g_asn1_octet_encode(ASN1_SCK *asn1, guchar ch)
 {
     if (asn1->pointer <= asn1->begin)
 	{
-		g_asn1ErrStatus = ASN1_ERR_ENC_FULL;
+		asn1->error = ASN1_ERR_ENC_FULL;
         return FALSE;
 	}
     *--(asn1->pointer) = ch;
@@ -158,7 +157,7 @@ g_asn1_octet_decode(ASN1_SCK *asn1, guchar *ch)
 {
     if (asn1->pointer >= asn1->end)
 	{
-		g_asn1ErrStatus = ASN1_ERR_DEC_EMPTY;
+		asn1->error = ASN1_ERR_DEC_EMPTY;
         return FALSE;
 	}
     *ch = *(asn1->pointer)++;
@@ -552,14 +551,14 @@ g_asn1_eoc_decode (ASN1_SCK *asn1, guchar   *eoc)
 	return FALSE;
       if (ch != 0x00)
 	{
-	  g_asn1ErrStatus = ASN1_ERR_DEC_EOC_MISMATCH;
+	  asn1->error = ASN1_ERR_DEC_EOC_MISMATCH;
 	  return FALSE;
 	}
       if (!g_asn1_octet_decode (asn1, &ch))
 	return FALSE;
       if (ch != 0x00)
 	{
-	  g_asn1ErrStatus = ASN1_ERR_DEC_EOC_MISMATCH;
+	  asn1->error = ASN1_ERR_DEC_EOC_MISMATCH;
 	  return FALSE;
 	}
       return TRUE;
@@ -568,7 +567,7 @@ g_asn1_eoc_decode (ASN1_SCK *asn1, guchar   *eoc)
     {
       if (asn1->pointer != eoc)
 	{
-	  g_asn1ErrStatus = ASN1_ERR_DEC_LENGTH_MISMATCH;
+	  asn1->error = ASN1_ERR_DEC_LENGTH_MISMATCH;
 	  return FALSE;
 	}
       return TRUE;
@@ -674,7 +673,7 @@ g_asn1_bool_decode ( ASN1_SCK *asn1, guchar *eoc, gboolean *bool)
     *bool = ch ? 1 : 0;
     if (asn1->pointer != eoc)
 	{
-		g_asn1ErrStatus = ASN1_ERR_DEC_LENGTH_MISMATCH;
+		asn1->error = ASN1_ERR_DEC_LENGTH_MISMATCH;
         return FALSE;
 	}
     return TRUE;
@@ -755,7 +754,7 @@ asn1_int_decode ( ASN1_SCK *asn1, guchar *eoc, gint *integer)
     {
         if (++len > sizeof (gint))
 		{
-			g_asn1ErrStatus = ASN1_ERR_DEC_BADVALUE;
+			asn1->error = ASN1_ERR_DEC_BADVALUE;
             return FALSE;
 		}
         if (!g_asn1_octet_decode (asn1, &ch))
@@ -841,7 +840,7 @@ g_asn1_long_decode ( ASN1_SCK *asn1, guchar *eoc, glong *integer)
     {
       if (++len > sizeof (glong))
 	{
-	  g_asn1ErrStatus = ASN1_ERR_DEC_BADVALUE;
+	  asn1->error = ASN1_ERR_DEC_BADVALUE;
 	  return FALSE;
 	}
       if (!g_asn1_octet_decode (asn1, &ch))
@@ -917,7 +916,7 @@ g_asn1_uint_decode ( ASN1_SCK *asn1, guchar *eoc, guint *integer)
     {
         if (++len > sizeof (guint))
 		{
-			g_asn1ErrStatus = ASN1_ERR_DEC_BADVALUE;
+			asn1->error = ASN1_ERR_DEC_BADVALUE;
             return FALSE;
 		}
         if (!g_asn1_octet_decode (asn1, &ch))
@@ -995,7 +994,7 @@ g_asn1_ulong_decode ( ASN1_SCK *asn1, guchar *eoc, gulong *integer)
     {
         if (++len > sizeof (gulong))
 		{
-			g_asn1ErrStatus = ASN1_ERR_DEC_BADVALUE;
+			asn1->error = ASN1_ERR_DEC_BADVALUE;
             return FALSE;
 		}
         if (!g_asn1_octet_decode (asn1, &ch))
@@ -1066,22 +1065,23 @@ g_asn1_bits_encode ( ASN1_SCK *asn1, guchar **eoc, guchar *bits,
  */
 
 gboolean 
-g_asn1_bits_decode ( ASN1_SCK *asn1, guchar *eoc, guchar *bits,
-		     guint size, guint *len, guchar *unused)
+g_asn1_bits_decode ( ASN1_SCK *asn1, guchar *eoc, guchar **bits,
+		     guint *len, guchar *unused)
 
 {
+    *bits = NULL;
     if (!g_asn1_octet_decode (asn1, unused))
         return FALSE;
     *len = 0;
+    *bits = g_malloc(eoc - asn1->pointer);
     while (asn1->pointer < eoc)
     {
-        if (++(*len) > size)
-	  {
-	    g_asn1ErrStatus = ASN1_ERR_DEC_BADVALUE;
-            return FALSE;
-	  }
         if (!g_asn1_octet_decode (asn1, (guchar *)bits++))
-	  return FALSE;
+          {
+            g_free(*bits);
+            *bits = NULL;
+	    return FALSE;
+          }
     }
     return TRUE;
 }
@@ -1099,23 +1099,24 @@ g_asn1_bits_decode ( ASN1_SCK *asn1, guchar *eoc, guchar *bits,
  *              Parameters:
  *              asn1:   pointer to ASN1 socket.
  *              eoc:    pointer to end of encoding.
- *              octets: pointer to begin of Octet String.
- *              len:    Length of Octet String in characters.
+ *              octets: pointer to begin of Octet GString.
  * RETURNS:     gboolean success.
  */
 
 gboolean 
-g_asn1_octets_encode ( ASN1_SCK *asn1, guchar **eoc, guchar *octets,
-		       guint len )
+g_asn1_octets_encode ( ASN1_SCK *asn1, guchar **eoc, guchar *octets, guint len)
 {
-    *eoc = asn1->pointer;
-    octets += len;
-    while (len-- > 0)
+  guchar *ptr;
+
+  *eoc = asn1->pointer;
+  ptr = octets + len;
+
+  while (len-- > 0)
     {
-        if (!g_asn1_octet_encode (asn1, *--octets))
-            return FALSE;
+      if (!g_asn1_octet_encode (asn1, *--ptr))
+        return FALSE;
     }
-    return TRUE;
+  return TRUE;
 }
 
 /*
@@ -1125,7 +1126,6 @@ g_asn1_octets_encode ( ASN1_SCK *asn1, guchar **eoc, guchar *octets,
  *                      ASN1_SCK *asn1,
  *                      guchar   *eoc,
  *                      guchar   *octets,
- *                      guint     size,
  *                      guint    *len
  *                  )
  * DESCRIPTION: Decodes Octet String.
@@ -1133,26 +1133,28 @@ g_asn1_octets_encode ( ASN1_SCK *asn1, guchar **eoc, guchar *octets,
  *              asn1: pointer to ASN1 socket.
  *              Eoc: pointer to end of encoding or 0 if
  *                   indefinite.
- *              Ots: pointer to begin of Octet String.
- *              OtsSze: Size of Octet String in characters.
- *              OtsLen: Length of Octet String in characters.
+ *              Ots: pointer to Object GString*.
  * RETURNS:     gboolean success.
  */
 
 gboolean 
-g_asn1_octets_decode ( ASN1_SCK *asn1, guchar *eoc, guchar *octets,
-		       guint size, guint *len)
+g_asn1_octets_decode ( ASN1_SCK *asn1, guchar *eoc, guchar **octets, guint *len)
 {
+  guchar *ptr;
+
+  *octets = NULL;
   *len = 0;
+  *octets = g_malloc (eoc - asn1->pointer);
+  ptr = *octets;
   while (asn1->pointer < eoc)
     {
-      if (++(*len) > size)
-	{
-	  g_asn1ErrStatus = ASN1_ERR_DEC_BADVALUE;
+      if (!g_asn1_octet_decode (asn1, (guchar *)ptr++))
+        {
+          g_free(*octets);
+          *octets = NULL;
 	  return FALSE;
-	}
-      if (!g_asn1_octet_decode (asn1, (guchar *)octets++))
-	return FALSE;
+        }
+      (*len)++;
     }
   return TRUE;
 }
@@ -1248,7 +1250,7 @@ g_asn1_oid_encode ( ASN1_SCK *asn1, guchar **eoc, gulong *oid,
   *eoc = asn1->pointer;
   if (len < 2)
     {
-      g_asn1ErrStatus = ASN1_ERR_ENC_BADVALUE;
+      asn1->error = ASN1_ERR_ENC_BADVALUE;
       return FALSE;
     }
   subid = oid [1] + oid [0] * 40;
@@ -1270,7 +1272,6 @@ g_asn1_oid_encode ( ASN1_SCK *asn1, guchar **eoc, gulong *oid,
  *                      ASN1_SCK *asn1,
  *                      char     *eoc,
  *                      gulong   *oid,
- *                      guint     size,
  *                      guint    *len
  *                  )
  * DESCRIPTION: Decodes Obect Identifier.
@@ -1285,44 +1286,55 @@ g_asn1_oid_encode ( ASN1_SCK *asn1, guchar **eoc, gulong *oid,
  */
 
 gboolean 
-g_asn1_oid_decode ( ASN1_SCK *asn1, guchar *eoc, gulong *oid, 
-		    guint size, guint *len)
+g_asn1_oid_decode ( ASN1_SCK *asn1, guchar *eoc, gulong **oid, 
+		    guint *len)
 {
   gulong subid;
-  
-  if (size < 2)
+  guint  size;
+  gulong *optr;
+
+  size = eoc - asn1->pointer + 1;
+  *oid = g_malloc(size * sizeof(gulong));
+  optr = *oid;
+ 
+  if (!g_asn1_subid_decode (asn1, &subid))
     {
-      g_asn1ErrStatus = ASN1_ERR_DEC_BADVALUE;
+      g_free(*oid);
+      *oid = NULL;
       return FALSE;
     }
-  if (!g_asn1_subid_decode (asn1, &subid))
-    return FALSE;
   if (subid < 40)
     {
-      oid [0] = 0;
-      oid [1] = subid;
+      optr [0] = 0;
+      optr [1] = subid;
     }
   else if (subid < 80)
     {
-      oid [0] = 1;
-      oid [1] = subid - 40;
+      optr [0] = 1;
+      optr [1] = subid - 40;
     }
   else
     {
-      oid [0] = 2;
-      oid [1] = subid - 80;
+      optr [0] = 2;
+      optr [1] = subid - 80;
     }
   *len = 2;
-  oid += 2;
+  optr += 2;
   while (asn1->pointer < eoc)
     {
       if (++(*len) > size)
 	{
-	  g_asn1ErrStatus = ASN1_ERR_DEC_BADVALUE;
+	  asn1->error = ASN1_ERR_DEC_BADVALUE;
+          g_free(*oid);
+          *oid = NULL;
 	  return FALSE;
 	}
-      if (!g_asn1_subid_decode (asn1, oid++))
-	return FALSE;
+      if (!g_asn1_subid_decode (asn1, optr++))
+        {
+          g_free(*oid);
+          *oid = NULL;
+	  return FALSE;
+        }
     }
   return TRUE;
 }
