@@ -1,7 +1,7 @@
 /*
  * GXSNMP -- An snmp mangament application
  * Copyright (C) 1998 Gregory McLean & Jochen Friedrich
- * Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group
+ * Beholder RMON ethernet network monitor, Copyright (C) 1993 DNPAP group
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,8 +45,6 @@ static gboolean g_snmp_request_encode(ASN1_SCK *asn1, SNMP_REQUEST *request);
 static gboolean g_snmp_request_decode(ASN1_SCK *asn1, SNMP_REQUEST *request);
 static gboolean g_snmp_trap_encode(ASN1_SCK *asn1, SNMP_V1_TRAP *trap);
 static gboolean g_snmp_trap_decode(ASN1_SCK *asn1, SNMP_V1_TRAP *trap);
-static gboolean g_snmp_pdu_encode(ASN1_SCK *asn1, SNMP_PDU *pdu, GSList *list);
-static gboolean g_snmp_pdu_decode(ASN1_SCK *asn1, SNMP_PDU *pdu, GSList **list);
 
 #define SNMP_IPA    0
 #define SNMP_CNT    1
@@ -118,7 +116,7 @@ static SNMP_CNV SnmpCnv [] =
  * RETURNS:     gboolean success.
  */
 
-gboolean  
+static gboolean  
 g_snmp_syntax2tag_cls ( guint *tag, guint *cls, gint syntax)
 {
     SNMP_CNV *cnv;
@@ -151,7 +149,7 @@ g_snmp_syntax2tag_cls ( guint *tag, guint *cls, gint syntax)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static gboolean 
 g_snmp_tag_cls2syntax ( guint tag, guint cls, gushort *syntax)
 {
     SNMP_CNV *cnv;
@@ -181,7 +179,7 @@ g_snmp_tag_cls2syntax ( guint tag, guint cls, gushort *syntax)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static gboolean 
 g_snmp_object_encode ( ASN1_SCK *asn1, SNMP_OBJECT *obj)
 {
     guint   cls, tag;
@@ -197,7 +195,8 @@ g_snmp_object_encode ( ASN1_SCK *asn1, SNMP_OBJECT *obj)
             break;
         case SNMP_OCTETSTR:
         case SNMP_OPAQUE:
-            if (!g_asn1_octets_encode (asn1, &end, obj->syntax.bufchr, obj->syntax_len))
+            if (!g_asn1_octets_encode (asn1, &end, obj->syntax.string, 
+                                       obj->syntax.lng))
                 return FALSE;
             break;
         case SNMP_NULL:
@@ -208,11 +207,13 @@ g_snmp_object_encode ( ASN1_SCK *asn1, SNMP_OBJECT *obj)
                 return FALSE;
             break;
         case SNMP_OBJECTID:
-            if (!g_asn1_oid_encode (asn1, &end, obj->syntax.bufint, obj->syntax_len))
+            if (!g_asn1_oid_encode (asn1, &end, obj->syntax.oid, 
+                                    obj->syntax_len))
                 return FALSE;
             break;
         case SNMP_IPADDR:
-            if (!g_asn1_octets_encode (asn1, &end, (guchar *)&obj->syntax.ulng, 4))
+            if (!g_asn1_octets_encode (asn1, &end, obj->syntax.string, 
+                                       obj->syntax_len))
                 return FALSE;
             break;
         case SNMP_COUNTER:
@@ -249,7 +250,7 @@ g_snmp_object_encode ( ASN1_SCK *asn1, SNMP_OBJECT *obj)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static gboolean 
 g_snmp_object_decode ( ASN1_SCK *asn1, SNMP_OBJECT *obj)
 {
     guint cls, con, tag;
@@ -263,8 +264,7 @@ g_snmp_object_decode ( ASN1_SCK *asn1, SNMP_OBJECT *obj)
         return FALSE;
     if (cls != ASN1_UNI || con != ASN1_PRI || tag != ASN1_OJI)
         return FALSE;
-    if (!g_asn1_oid_decode (asn1, end, obj->id, SNMP_SIZE_OBJECTID, 
-        &obj->id_len))
+    if (!g_asn1_oid_decode (asn1, end, &obj->id, &obj->id_len))
         return FALSE;
     if (!g_asn1_header_decode (asn1, &end, &cls, &con, &tag))
         return FALSE;
@@ -283,8 +283,8 @@ g_snmp_object_decode ( ASN1_SCK *asn1, SNMP_OBJECT *obj)
             break;
         case SNMP_OCTETSTR:
         case SNMP_OPAQUE:
-            if (!g_asn1_octets_decode (asn1, end, obj->syntax.bufchr, 
-                                       SNMP_SIZE_BUFCHR, &obj->syntax_len))
+            if (!g_asn1_octets_decode (asn1, end, &obj->syntax.string,
+                                       &obj->syntax_len))
                 return FALSE;
             break;
         case SNMP_NULL:
@@ -295,16 +295,15 @@ g_snmp_object_decode ( ASN1_SCK *asn1, SNMP_OBJECT *obj)
                 return FALSE;
             break;
         case SNMP_OBJECTID:
-            if (!g_asn1_oid_decode (asn1, end, obj->syntax.bufint, 
-                                    SNMP_SIZE_BUFINT, &obj->syntax_len))
+            if (!g_asn1_oid_decode (asn1, end, &obj->syntax.oid, 
+                                    &obj->syntax_len))
                 return FALSE;
             break;
         case SNMP_IPADDR:
-            if (!g_asn1_octets_decode (asn1, end, 
-                                      (guchar *)&obj->syntax.ulng, 4,
-                                      &obj->syntax_len))
+            if (!g_asn1_octets_decode (asn1, end, &obj->syntax.string,
+                                       &obj->syntax_len))
                 return FALSE;
-            if (obj->syntax_len != 4)
+            if ((obj->syntax_len != 4) && (obj->syntax_len != 16))
                 return FALSE;
             break;
         case SNMP_COUNTER:
@@ -333,7 +332,7 @@ g_snmp_object_decode ( ASN1_SCK *asn1, SNMP_OBJECT *obj)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static gboolean 
 g_snmp_list_encode ( ASN1_SCK *asn1, GSList *list)
 {
     guchar *eoc;
@@ -364,7 +363,7 @@ g_snmp_list_encode ( ASN1_SCK *asn1, GSList *list)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static gboolean 
 g_snmp_list_decode (ASN1_SCK *asn1, GSList **list)
 {
     guint cls, con, tag;
@@ -399,7 +398,7 @@ g_snmp_list_decode (ASN1_SCK *asn1, GSList **list)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static gboolean 
 g_snmp_request_encode ( ASN1_SCK *asn1, SNMP_REQUEST *request)
 {
     guchar *end;
@@ -430,7 +429,7 @@ g_snmp_request_encode ( ASN1_SCK *asn1, SNMP_REQUEST *request)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static gboolean 
 g_snmp_request_decode ( ASN1_SCK *asn1, SNMP_REQUEST *request)
 {
     guint cls, con, tag;
@@ -467,7 +466,7 @@ g_snmp_request_decode ( ASN1_SCK *asn1, SNMP_REQUEST *request)
  * DESCRIPTION: Encodes a trap PDU in ASN1.
  * RETURNS:     gboolean success.
  */
-gboolean 
+static gboolean 
 g_snmp_trap_encode ( ASN1_SCK *asn1, SNMP_V1_TRAP *trap)
 {
     guchar *end;
@@ -506,24 +505,24 @@ g_snmp_trap_encode ( ASN1_SCK *asn1, SNMP_V1_TRAP *trap)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static gboolean 
 g_snmp_trap_decode ( ASN1_SCK *asn1, SNMP_V1_TRAP *trap)
 {
-    guint cls, con, tag;
+    guint cls, con, tag, len;
     guchar *end;
 
     if (!g_asn1_header_decode (asn1, &end, &cls, &con, &tag))
         return FALSE;
     if (cls != ASN1_UNI || con != ASN1_PRI || tag != ASN1_OJI)
         return FALSE;
-    if (!g_asn1_oid_decode (asn1, end, trap->id, sizeof(trap->id)/sizeof(trap->id[0]), &trap->id_len))
+    if (!g_asn1_oid_decode (asn1, end, &trap->id, &trap->id_len))
         return FALSE;
     if (!g_asn1_header_decode (asn1, &end, &cls, &con, &tag))
         return FALSE;
     if (!((cls == ASN1_APL && con == ASN1_PRI && tag == SNMP_IPA) ||
 		(cls == ASN1_UNI && con == ASN1_PRI && tag == ASN1_OTS)))	/* needed for Banyan */
         return FALSE;
-    if (!g_asn1_octets_decode (asn1, end, (guchar *)&(trap->ip_address), 4, &tag))
+    if (!g_asn1_octets_decode (asn1, end, (guchar **)&trap->ip_address, &len))
         return FALSE;
     if (!g_asn1_header_decode (asn1, &end, &cls, &con, &tag))
         return FALSE;
@@ -553,7 +552,6 @@ g_snmp_trap_decode ( ASN1_SCK *asn1, SNMP_V1_TRAP *trap)
  *                  (
  *                      ASN1_SCK     *asn1,
  *                      SNMP_PDU     *pdu,
- *                      GSList       *list
  *                  )
  * DESCRIPTION: Encodes a PDU in ASN1.
  *              Pdu is a union of snmp_ror and snmp_trp.
@@ -561,24 +559,26 @@ g_snmp_trap_decode ( ASN1_SCK *asn1, SNMP_V1_TRAP *trap)
  */
 
 gboolean 
-g_snmp_pdu_v1_encode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList *list)
+g_snmp_pdu_v1_encode ( ASN1_SCK *asn1, SNMP_PDU *pdu)
 {
     guchar *eoc;
 
     if (!g_asn1_eoc_encode (asn1, &eoc))
         return FALSE;
-    if (!g_snmp_list_encode (asn1, list))
-        return FALSE;
     switch (pdu->type)
     {
         case SNMP_PDU_GET:
         case SNMP_PDU_NEXT:
         case SNMP_PDU_RESPONSE:
         case SNMP_PDU_SET:
+            if (!g_snmp_list_encode (asn1, pdu->request.variables))
+                return FALSE;
             if (!g_snmp_request_encode (asn1, &pdu->request))
                 return FALSE;
             break;
         case SNMP_PDU_TRAP1:
+            if (!g_snmp_list_encode (asn1, pdu->trap.variables))
+                return FALSE;
            if (!g_snmp_trap_encode (asn1, &pdu->trap))
                 return FALSE;
             break;
@@ -596,7 +596,6 @@ g_snmp_pdu_v1_encode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList *list)
  *                  (
  *                      ASN1_SCK     *asn1,
  *                      SNMP_PDU     *pdu,
- *                      GSList       *list
  *                  )
  * DESCRIPTION: Encodes a PDU in ASN1.
  *              Pdu is a union of snmp_ror and snmp_trp.
@@ -604,14 +603,12 @@ g_snmp_pdu_v1_encode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList *list)
  */
 
 gboolean 
-g_snmp_pdu_v2_encode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList *list)
+g_snmp_pdu_v2_encode ( ASN1_SCK *asn1, SNMP_PDU *pdu)
 {
     guchar *eoc;
 
     if (!g_asn1_eoc_encode (asn1, &eoc))
         return FALSE;
-    if (!g_snmp_list_encode (asn1, list))
-        return FALSE;
     switch (pdu->type)
     {
         case SNMP_PDU_GET:
@@ -621,6 +618,8 @@ g_snmp_pdu_v2_encode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList *list)
 	case SNMP_PDU_BULK:
 	case SNMP_PDU_INFORM:
 	case SNMP_PDU_TRAP2:
+            if (!g_snmp_list_encode (asn1, pdu->request.variables))
+                return FALSE;
             if (!g_snmp_request_encode (asn1, &pdu->request))
                 return FALSE;
             break;
@@ -638,7 +637,6 @@ g_snmp_pdu_v2_encode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList *list)
  *                  (
  *                      ASN1_SCK     *asn1,
  *                      SNMP_PDU     *pdu
- *                      GSList      **list
  *                  )
  * DESCRIPTION: Decodes a PDU from ASN1.
  *              pdu is a union of snmp_ror and snmp_trp.
@@ -646,7 +644,7 @@ g_snmp_pdu_v2_encode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList *list)
  */
 
 gboolean 
-g_snmp_pdu_v1_decode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList **list)
+g_snmp_pdu_v1_decode ( ASN1_SCK *asn1, SNMP_PDU *pdu)
 {
     guint cls, con;
     guchar *eoc;
@@ -663,16 +661,18 @@ g_snmp_pdu_v1_decode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList **list)
         case SNMP_PDU_SET:
             if (!g_snmp_request_decode (asn1, &pdu->request))
                 return FALSE;
+            if (!g_snmp_list_decode (asn1, &pdu->request.variables))
+                return FALSE;
             break;
         case SNMP_PDU_TRAP1:
             if (!g_snmp_trap_decode (asn1, &pdu->trap))
                 return FALSE;
+            if (!g_snmp_list_decode (asn1, &pdu->trap.variables))
+                return FALSE;
             break;
         default:
             return FALSE;
     }
-    if (!g_snmp_list_decode (asn1, list))
-        return FALSE;
     if (!g_asn1_eoc_decode (asn1, eoc))
         return FALSE;
     return TRUE;
@@ -684,7 +684,6 @@ g_snmp_pdu_v1_decode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList **list)
  *                  (
  *                      ASN1_SCK     *asn1,
  *                      SNMP_PDU     *pdu
- *                      GSList      **list
  *                  )
  * DESCRIPTION: Decodes a PDU from ASN1.
  *              pdu is snmp_ror.
@@ -692,7 +691,7 @@ g_snmp_pdu_v1_decode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList **list)
  */
 
 gboolean 
-g_snmp_pdu_v2_decode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList **list)
+g_snmp_pdu_v2_decode ( ASN1_SCK *asn1, SNMP_PDU *pdu)
 {
     guint cls, con;
     guchar *eoc;
@@ -712,230 +711,15 @@ g_snmp_pdu_v2_decode ( ASN1_SCK *asn1, SNMP_PDU *pdu, GSList **list)
         case SNMP_PDU_TRAP2:
             if (!g_snmp_request_decode (asn1, &pdu->request))
                 return FALSE;
+            if (!g_snmp_list_decode (asn1, &pdu->request.variables))
+                return FALSE;
             break;
         default:
             return FALSE;
     }
-    if (!g_snmp_list_decode (asn1, list))
-        return FALSE;
     if (!g_asn1_eoc_decode (asn1, eoc))
         return FALSE;
     return TRUE;
 }
 
-gboolean 
-g_snmp_message_encode ( ASN1_SCK *asn1, SNMP_PDU *pdu, SNMP_AUTH *auth,
-			guint version, GSList *list)
-
-{
-  guchar *eoc, *end;
-    
-  if (!g_asn1_eoc_encode (asn1, &eoc))
-    return FALSE;
-  switch(version)
-    {
-      case SNMP_V1:
-        if (!g_snmp_pdu_v1_encode (asn1, pdu, list))
-          return FALSE;
-        if (auth->type != AUTH_COMMUNITY)
-          return FALSE;
-        if (!g_asn1_octets_encode (asn1, &end, auth->name, auth->nlen))
-          return FALSE;
-        if (!g_asn1_header_encode (asn1, end, ASN1_UNI, ASN1_PRI, ASN1_OTS))
-          return FALSE;
-        break;
-      case SNMP_V2C:
-        if (!g_snmp_pdu_v2_encode (asn1, pdu, list))
-          return FALSE;
-        if (auth->type != AUTH_COMMUNITY)
-          return FALSE;
-        if (!g_asn1_octets_encode (asn1, &end, auth->name, auth->nlen))
-          return FALSE;
-        if (!g_asn1_header_encode (asn1, end, ASN1_UNI, ASN1_PRI, ASN1_OTS))
-          return FALSE;
-        break;
-      default:
-        return FALSE;
-    }
-  if (!g_asn1_uint_encode (asn1, &end, version))
-    return FALSE;
-  if (!g_asn1_header_encode (asn1, end, ASN1_UNI, ASN1_PRI, ASN1_INT))
-    return FALSE;
-  if (!g_asn1_header_encode (asn1, eoc, ASN1_UNI, ASN1_CON, ASN1_SEQ))
-    return FALSE;
-  return TRUE;
-}
-
-gboolean 
-g_snmp_message_decode ( ASN1_SCK *asn1, SNMP_PDU *pdu, SNMP_AUTH *auth,
-			guint *version, GSList **list)
-{
-  guint cls, con, tag;
-  guchar *eoc, *end;
-
-  if (!g_asn1_header_decode (asn1, &eoc, &cls, &con, &tag))
-    return FALSE;
-  if (cls != ASN1_UNI || con != ASN1_CON || tag != ASN1_SEQ)
-    return FALSE;
-  if (!g_asn1_header_decode (asn1, &end, &cls, &con, &tag))
-    return FALSE;
-  if (cls != ASN1_UNI || con != ASN1_PRI || tag != ASN1_INT)
-    return FALSE;
-  if (!g_asn1_uint_decode (asn1, end, version))
-    return FALSE;
-  if (!g_asn1_header_decode (asn1, &end, &cls, &con, &tag))
-    return FALSE;
-  switch(*version)
-    {
-      case SNMP_V1:
-        auth->type = AUTH_COMMUNITY;
-        if (cls != ASN1_UNI || con != ASN1_PRI || tag != ASN1_OTS)
-          return FALSE;
-        if (!g_asn1_octets_decode (asn1, end, auth->name, 
-            AUTH_COMLEN, &(auth->nlen)))
-          return FALSE;
-        if (!g_snmp_pdu_v1_decode (asn1, pdu, list))
-          return FALSE;
-        break;
-      case SNMP_V2C:
-        auth->type = AUTH_COMMUNITY;
-        if (cls != ASN1_UNI || con != ASN1_PRI || tag != ASN1_OTS)
-          return FALSE;
-        if (!g_asn1_octets_decode (asn1, end, auth->name, 
-            AUTH_COMLEN, &(auth->nlen)))
-          return FALSE;
-        if (!g_snmp_pdu_v2_decode (asn1, pdu, list))
-          return FALSE;
-        break;
-      default:
-        return FALSE;
-    }
-  if (!g_asn1_eoc_decode (asn1, eoc))
-    return FALSE;
-  return TRUE;
-}
-
-/*
- * NAME:        g_snmp_encode                                  [API]
- * SYNOPSIS:    gboolean g_snmp_encode
- *                  (
- *                      guchar     **snmp, 
- *                      guint       *snmp_len,
- *                      SNMP_PDU    *pdu, 
- *                      SNMP_AUTH   *auth, 
- *                      GSList      *list
- *                  )
- * DESCRIPTION: Encodes a request, response or trap in ASN1.
- * RETURNS:     gboolean success.
- */
-gboolean 
-g_snmp_encode(guchar **snmp, guint *snmp_len, SNMP_PDU *pdu,
-                       SNMP_AUTH *auth, guint version, GSList *list)
-{
-  ASN1_SCK asn1;
-
-  g_snmpErrStatus = SNMP_NOERROR;
-  g_snmpErrIndex = 0;
-
-  g_asn1_open (&asn1, *snmp, *snmp_len, ASN1_ENC);
-  if (!g_snmp_message_encode(&asn1, pdu, auth, version, list))
-    {
-      if (g_snmpErrStatus == SNMP_NOERROR)
-        {
-          switch (g_asn1ErrStatus)
-            {
-              case ASN1_ERR_ENC_FULL:
-                g_snmpErrStatus = SNMP_TOOBIG;
-                break;
-              case ASN1_ERR_NOERROR:
-              case ASN1_ERR_DEC_EMPTY:
-              case ASN1_ERR_DEC_EOC_MISMATCH:
-              case ASN1_ERR_DEC_LENGTH_MISMATCH:
-              case ASN1_ERR_DEC_BADVALUE:
-              case ASN1_ERR_ENC_BADVALUE:
-              default:
-                g_snmpErrStatus = SNMP_GENERROR;
-                break;
-
-            }
-        }
-      return FALSE;
-    }
-  g_asn1_close (&asn1, snmp, snmp_len);
-
-  return TRUE;
-}
-
-/*
- * NAME:        g_snmp_decode                             [API]
- * SYNOPSIS:    gboolean g_snmp_decode
- *                  (
- *                      guchar     *snmp, 
- *                      guint       len,
- *                      SNMP_PDU   *pdu,
- *                      SNMP_AUTH  *auth,
- *                      guint      *version,
- *                      GSList    **list
- *                  )
- * DESCRIPTION: Decodes a request, response or trap from ASN1.
- * RETURNS:     unsigned short success.
- */
-
-gboolean 
-g_snmp_decode(guchar *snmp, guint snmp_len, SNMP_PDU *pdu,
-	      SNMP_AUTH *auth, guint *version, GSList **list)
-{
-  ASN1_SCK asn1;
-
-  g_snmpErrStatus = SNMP_NOERROR;
-  g_snmpErrIndex = 0;
-
-  g_asn1_open (&asn1, snmp, snmp_len, ASN1_DEC);
-  if (!g_snmp_message_decode(&asn1, pdu, auth, version, list))
-    {
-      if (g_snmpErrStatus == SNMP_NOERROR)
-        {
-          switch (g_asn1ErrStatus)
-            {
-              case ASN1_ERR_DEC_BADVALUE:
-              case ASN1_ERR_DEC_EOC_MISMATCH:
-              case ASN1_ERR_DEC_LENGTH_MISMATCH:
-              case ASN1_ERR_DEC_EMPTY:
-                g_snmpErrStatus = SNMP_BADVALUE;
-                break;
-              case ASN1_ERR_ENC_FULL:
-              case ASN1_ERR_NOERROR:
-              case ASN1_ERR_ENC_BADVALUE:
-              default:
-                g_snmpErrStatus = SNMP_GENERROR;
-                break;
-
-            }
-        }
-      return FALSE;
-    }
-    g_asn1_close (&asn1, &snmp, &snmp_len);
-    return TRUE;
-}
-
-void
-g_snmp_error (guchar *ptr)
-{
-  switch (g_snmpErrStatus)
-    {
-      case SNMP_BADVALUE:
-        ptr = "SNMP Failed: Bad Value.";
-        break;
-      case SNMP_TOOBIG:
-        ptr = "SNMP Failed: PDU too big.";
-        break;
-      case SNMP_GENERROR:
-        ptr = "SNMP Failed: Generic Error.";
-        break;
-      default:
-        ptr = "Unknown snmp error.";
-        break;
-    }
-}
-
 /* EOF */
