@@ -66,6 +66,7 @@ int IsLeapYear (int year);
 int RepaintCalendar (GtkWidget * widget, GdkEventExpose * event);
 int GetDayOfWeek (int day, int month, int year);
 void DrawGlassWin (GdkWindow *win, int xs, int ys, int w, int h);
+void set_scrollbar_size (int num_lines);
 GtkWidget *new_pixmap_from_data(char **xpm_data, GdkWindow *w, GdkColor *b);
 DateMark *FindPrevMark ();
 DateMark *FindNextMark ();
@@ -93,6 +94,7 @@ extern GdkGC *gc;
 extern Log *curlog, *loglist[];
 extern int numlogs, curlognum;
 extern char *month[12];
+extern GtkWidget *main_win_scrollbar;
 
 GtkWidget *CalendarDialog = NULL;
 GdkDrawable *CalendarCanvas;
@@ -422,6 +424,10 @@ HandleMouseButton( GtkWidget * cv, GdkEventButton * event)
        curlog->curmark = thisdate->mark;
        MoveToMark (curlog);
        curlog->firstline = 0;
+       curlog->ln = curlog->curmark->ln;
+       gtk_adjustment_set_value ( GTK_RANGE(main_win_scrollbar)->adjustment, 
+				  curlog->ln);
+       /* set_scrollbar_size (curlog->lstats.numlines); */
        log_repaint(NULL, NULL);
      }
 }
