@@ -152,5 +152,10 @@ CreateConfig()
   gdk_font_unref (cs->font);
   cs->font = gdk_font_load (HELVETICA_10_FONT);
 
+  /* Set paths */
+  newcfg->regexp_db_path = NULL;
+  newcfg->descript_db_path = NULL;
+  newcfg->action_db_path = NULL;
+
   return (ConfigData *) newcfg;
 }
